use rayon::prelude::*;
use regex::Regex;
use std::collections::HashMap;
use std::fs;
use std::sync::{Arc, Mutex};

pub fn solve() {
    let path = "res/03/input";

    let contents = fs::read_to_string(path).expect("I/O error, wrong path?");
    //let contents = io::BufReader::new(fs::File::open(path).expect("I/O error, wrong path?"));

    let grid: Vec<Vec<char>> = contents
        .lines()
        .map(|l| l.chars().collect::<Vec<char>>())
        .collect();

    let possible_gears: Arc<Mutex<HashMap<(usize, usize), Vec<usize>>>> =
        Arc::new(Mutex::new(HashMap::new()));

    let result: usize = contents
        .lines()
        .enumerate()
        .par_bridge()
        .map(|(yusize, line)| {
            let re = Regex::new(r"(\d+)").unwrap();
            let y = yusize as i32;

            let mut nums = Vec::<usize>::new();

            for (_, mat) in re.find_iter(line).enumerate() {
                if let Ok(number) = mat.as_str().parse::<usize>() {
                    let x = mat.start() as i32;
                    let len = number.to_string().len() as i32;
                    let mut pos_to_check = vec![
                        (x - 1, y - 1),
                        (x - 1, y),
                        (x - 1, y + 1),
                        (x, y - 1),
                        (x, y + 1),
                        (x + len, y),
                    ];
                    for i in 0..len {
                        pos_to_check.push((x + 1 + i, y - 1));
                        pos_to_check.push((x + 1 + i, y + 1));
                    }

                    if pos_to_check
                        .iter()
                        .filter(|(x, y)| {
                            *x >= 0 && *y >= 0 && *y < grid.len() as i32 && *x < line.len() as i32
                        })
                        .map(|(x, y)| {
                            if grid[*y as usize][*x as usize] == '*' {
                                let mut possible_gears_mut = possible_gears.lock().unwrap();
                                possible_gears_mut
                                    .entry((*x as usize, *y as usize))
                                    .or_insert(vec![])
                                    .push(number);
                            }
                            (x, y)
                        })
                        .any(|(x, y)| {
                            !grid[*y as usize][*x as usize].is_digit(10)
                                && grid[*y as usize][*x as usize] != '.'
                        })
                    {
                        nums.push(number)
                    }
                }
            }
            nums.iter().sum::<usize>()
        })
        .sum();

    println!("Result 1: {}", result);

    let possible_gears_mut = possible_gears.lock().unwrap();
    let result: usize = possible_gears_mut
        .iter()
        .filter(|(_, vec)| vec.len() == 2)
        .map(|(_, vec)| vec.iter().product::<usize>())
        .sum();

    println!("Result 2: {}", result);
}
